#include <iostream>

#include "misc.hpp"
#include "learner.hpp"
#include "team.hpp"
#include "point.hpp"
#include "env.hpp"
#include "scm.hpp"

using namespace std;

/* TODO
   - Make sure there is at least one (point) individual of each class
   upon selection, not during generation. This should be
   faster. However, if you keep the last individual of a given class
   just because it's the last individual, it may linger even though it
   is useless -- it may therefore make sense to allow the class to
   become 'extinct' and then try to find a better individual of that
   class by sampling (making sure you get that class in there then).
   - Use Andy's score to select individual at the end of training.
 */

int main(int argc, 
	 char **argv)
{
  /* Argument must specify parameter file. */
  
  if(argc != 2)
    die(__FILE__, __FUNCTION__, __LINE__, "bad arguments");

  scm *model;
  map < string, string > args;
  map < string, string > :: iterator ariter;
  
  /* Get arguments. */

  cout << readMap(argv[1], args) << " args read" << endl;

  /* Seed random number generator. */

  if((ariter = args.find("seed")) == args.end())
    die(__FILE__, __FUNCTION__, __LINE__, "cannot find arg seed");

  srand48(stringToInt(ariter->second));

  /* Select environment type and run associated scm. */

  if((ariter = args.find("envType")) == args.end())
    die(__FILE__, __FUNCTION__, __LINE__, "cannot find arg envType");
  
  if(ariter->second == "datasetEnv")
    {
      model = new scmExplicit(args);
      model->go();
      delete model;
    }
  else
    {
      die(__FILE__, __FUNCTION__, __LINE__, "bad arg envType");
    }
 
  return 0; 
}


/* 
   - Using deterministic crowding on the team side and
   Pareto-front-based selection on the point side does not makes
   sense. This is because points are evaluated based on how they
   distinguish between all n teams (i.e., all n*n possible comparisons
   are considered) whereas in the team population offspring teams
   compete only with the parent teams. This implies that point fitness
   should consider only distinctions between parents and their
   offspring.

   - Using deterministic crowding on the point side may result in the
   maintenance of niches on the team side. If a single team does well
   on a particular point and no other team does well on that point
   (i.e., that team represents a niche), then the outcome of that team
   on that point will be high and the outcomes of all other teams will
   be low. Using Pareto-based selection, that team will remain in the
   population (i.e., the niche will be maintained) and because the
   point makes a distinction between that team and all other teams
   that point will also remain in the population.

   - Generating a balanced point subset may actually hurt points of
   minority classes. Consider this setup
   
   instance  dist. vector  class
   
   0         1000          0
   1         0110          1
   2         0110          1
   3         0110          1

   where dist is the distinction vector for patterns 0 through 3.
   Note that since pattern 0 is of a different class, it is likely to
   make distinctions different from those made by patterns 1 through 3
   (as shown). Assume class 1 is the majority class and that all four
   points fall in the pareto front and you must select two of
   them. The competitive fitness sharing score for pattern 0 is 1
   while for the other three patterns it's 2/3. If you force in a
   pattern of class 0 in place of pattern 3 by using balanced
   selection it is likely that it will make the same distinctions as
   pattern 0 so now patterns of class 0 have a score of 1/2 while
   patterns of class 1 still have a score of 2/2 and are favoured for
   selection. Therefore, using balanced subset generation actually
   hurts patterns of the minority classes.
 */
